/*****************************************************************
**
**	@(#)	updater.c
**
*****************************************************************/

#include "config.h"
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <assert.h>

#include <ldns/ldns.h>

#define extern
#include "updater.h"
#undef extern
#include "ddns.h"
#include "misc.h"
#include "debug.h"

ldns_resolver *getlocalresolver (uint16_t port, ldns_tsig_credentials *tsig_cred)
{
	ldns_resolver	*res;
	ldns_rr_list	*nsiplist;
	ldns_rr		*nsrr;
	char		str[254+1];

	// class = LDNS_RR_CLASS_IN;
	if ( port == 0 )
		port = 53;

	if ( (res = ldns_resolver_new()) == NULL )
		goto bad;
	ldns_resolver_set_port (res, port);
	        /* TSIG key data available? Copy into the resolver. */
	if ( tsig_cred )
	{
		ldns_resolver_set_tsig_algorithm (res, ldns_tsig_algorithm (tsig_cred));
		ldns_resolver_set_tsig_keyname (res, ldns_tsig_keyname (tsig_cred));
		ldns_resolver_set_tsig_keydata (res, ldns_tsig_keydata (tsig_cred));
	}

	if ( (nsiplist = ldns_rr_list_new()) == NULL )
		goto bad;

	snprintf (str, sizeof (str), ". IN AAAA ::1");
	if ( ldns_rr_new_frm_str (&nsrr, str, 0, NULL, NULL) != LDNS_STATUS_OK )
		goto bad;
	ldns_rr_list_push_rr (nsiplist, nsrr);
	snprintf (str, sizeof (str), ". IN A 127.0.0.1");
	if ( ldns_rr_new_frm_str (&nsrr, str, 0, NULL, NULL) != LDNS_STATUS_OK )
		goto bad;
	ldns_rr_list_push_rr (nsiplist, nsrr);

	(void) ldns_resolver_push_nameserver_rr_list (res, nsiplist);
	ldns_rr_list_deep_free (nsiplist);

	ldns_resolver_set_random (res, false);

	return res;              /* return the resolver object for the update message */

bad:    /* cleanup */
	if ( res ) ldns_resolver_deep_free (res);

        return NULL;
}

int	sendupdate (ldns_resolver *res, ldns_rdf *zone_rdf, ldns_rr_list *pr_rrlist, ldns_rr_list *up_rrlist, ldns_tsig_credentials *tsig_cred)
{
	int		ret = 0;
	ldns_pkt	*u_pkt;
	ldns_pkt	*r_pkt;

        /* Create update packet. */
        u_pkt = ldns_update_pkt_new (zone_rdf, LDNS_RR_CLASS_IN, pr_rrlist, up_rrlist, NULL);
	zone_rdf = NULL;
        if ( !u_pkt && up_rrlist )
                ldns_rr_list_deep_free (up_rrlist);

	dbg_line();
	ldns_pkt_set_random_id (u_pkt);

	dbg_line();
        /* Add TSIG */
	if ( tsig_cred && (ret = ldns_update_pkt_tsig_add (u_pkt, res)) != LDNS_STATUS_OK )
		fatal ("Couldn't sign update packet with tsig credentials\n");

	/* send packet */
	dbg_line();
	if ( (ret = ldns_resolver_send_pkt (&r_pkt, res, u_pkt)) != LDNS_STATUS_OK )
		goto cleanup;

	dbg_line();
	ldns_pkt_free (u_pkt);
	if ( !r_pkt )
		goto cleanup;

	dbg_line();
	/* get return code */
	if ( (ret = ldns_pkt_get_rcode (r_pkt)) != LDNS_RCODE_NOERROR )
	{
		ldns_lookup_table	*t;
		
		t = ldns_lookup_by_id (ldns_rcodes, (int)ldns_pkt_get_rcode (r_pkt));
		if ( t )
			fatal ("UPDATE error: response was %s\n", t->name);
		else
			fatal ("UPDATE error: response was (%d)\n", ldns_pkt_get_rcode(r_pkt));
	}
	ldns_pkt_free (r_pkt);
	ldns_resolver_deep_free (res);

	return 1;

cleanup:
	error ("LDNS error (%d): %s \n", ret, ldns_get_errorstr_by_id (ret));
	if ( res )
		ldns_resolver_deep_free (res);
	if ( u_pkt )
		ldns_pkt_free (u_pkt);
	if ( zone_rdf )
		ldns_rdf_deep_free (zone_rdf);

	return 0;
}
