/*****************************************************************
**
**	@(#) zkt.c  -- A library for managing a list of dns zone files. 
**
**	Copyright (c) 2005 - 2008, Holger Zuleger HZnet. All rights reserved.
**
**	This software is open source.
**
**	Redistribution and use in source and binary forms, with or without
**	modification, are permitted provided that the following conditions
**	are met:
**
**	Redistributions of source code must retain the above copyright notice,
**	this list of conditions and the following disclaimer.
**
**	Redistributions in binary form must reproduce the above copyright notice,
**	this list of conditions and the following disclaimer in the documentation
**	and/or other materials provided with the distribution.
**
**	Neither the name of Holger Zuleger HZnet nor the names of its contributors may
**	be used to endorse or promote products derived from this software without
**	specific prior written permission.
**
**	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
**	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
**	TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
**	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
**	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
**	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
**	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
**	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
**	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
**	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
**	POSSIBILITY OF SUCH DAMAGE.
**
*****************************************************************/
# include <stdio.h>
# include <string.h>
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
# include "dki.h"
# include "misc.h"
# include "strlist.h"
# include "zconf.h"
# include "domaincmp.h"
# include "tcap.h"
#define extern
# include "zkt.h"
#undef extern

extern	char	*labellist;
extern	int	headerflag;
extern	int	printtime;
extern	int	isotime;
extern	int	alltimer;
extern	int	kskflag;
extern	int	zskflag;
extern	int	pathflag;
extern	int	ljustflag;
extern	int	outputformat;
extern	int	width;

#if 0
# define NAME_FL	"%33.33s"
# define NAME_FR	"%-33.33s"
#else
# define NAME_FL	"%25.25s"
# define NAME_FR	"%-25.25s"
#endif

static	void	printkeyinfo (time_t reftime, const dki_t *dkp, const char *oldpath)
{
	int	len;
	int	i;

	len = (3 * outputformat) + 1;
	if ( dkp == NULL )	/* print headline */
	{
		if ( !headerflag )
			return;
		tc_attr (stdout, TC_BOLD, 1);
		printf ("%-*.*s %5s %3s %3.3s %-7s", width, width,
			"Keyname", "Tag", "Typ", "Status", "Algorit");

		for ( i = DKT_CREATE; i <= DKT_DELETE; i++ )
			if ( i == printtime )
				printf (" %s", dki_timelabel2str (i, 17));
			else
				printf (" %s", dki_timelabel2str (i, len));

		if ( alltimer )
			for ( i = DKT_REVOKE; i < DKT_MAX; i++ )
				printf (" %s", dki_timelabel2str (i, len));

		tc_attr (stdout, TC_BOLD, 0);
		putchar ('\n');

		return;
	}

	/* TODO: use next line if dname is dynamically allocated */
	/* if ( pathflag && dkp->dname && strcmp (oldpath, dkp->dname) != 0 ) */
	if ( pathflag && strcmp (oldpath, dkp->dname) != 0 )
		printf ("%s/\n", dkp->dname);

	if ( (kskflag && dki_isksk (dkp)) || (zskflag && dki_iszsk (dkp)) || (kskflag && zskflag) )
	{
		int	color;
		const	char	*str;

		if ( ljustflag )
			printf ("%-*.*s ", width, width, dkp->name);
		else
			printf ("%*.*s ", width, width, dkp->name);
		printf ("%05d ", dkp->tag);

		color = TC_NONE;
		str = "ZSK";
		if ( dki_isksk (dkp) )
		{
			color = TC_CYAN;
			str = "KSK";
		}
		else if ( dki_iskey (dkp) )
		{
			color = TC_YELLOW;
			str = "KEY";
		}
		tc_attr (stdout, color, 1);
		printf ("%3s ", str);
		tc_attr (stdout, color, 0);

		str = dki_statusstr (dkp);
		if ( *str == 'a' )
			color = TC_GREEN;
		else if ( *str == 'p' )
			color = TC_BLUE;
		else if ( *str == 'i' || *str == 'd'  )
			color = TC_RED;
		else
			color = TC_BLACK;

		tc_attr (stdout, color, 1);
		printf ("%-3.3s ", str );
		tc_attr (stdout, color, 0);

		printf ("%-7s", dki_algo2sstr(dkp->algo));

		if ( printtime >= 0 )	/* all timer values will be relative to "printtime" */
			reftime = dkp->t[printtime];

		for ( i = DKT_CREATE; i <= DKT_DELETE; i++ )	/* loop through the basic timer fields */
			if ( i == printtime )
				if ( isotime )
					printf ("  %s  ", time2isostr (dkp->t[i], 's'));
				else
					printf (" %s", time2str (dkp->t[i], 'm'));
			else
				printf (" %s", timediff2str (reftime, dkp->t[i], NULL, len+1));
					
		if ( alltimer )
			for ( i = DKT_REVOKE; i < DKT_MAX; i++ )
				printf (" %s", timediff2str (reftime, dkp->t[i], NULL, len+1));

		putchar ('\n');
	}
}

#if defined(USE_TREE) && USE_TREE
static	time_t	reftime_for_list_key;
static	void	list_key (const dki_t **nodep, const VISIT which, int depth)
{
	const	dki_t	*dkp;
	static	const	char	*oldpath = "";

	if ( nodep == NULL )
		return;
//fprintf (stderr, "listkey %d %d %s\n", which, depth, dkp->name);

	if ( which == INORDER || which == LEAF )
	{
		dkp = *nodep;
		while ( dkp )	/* loop through list */
		{
			if ( labellist == NULL || isinlist (dkp->name, labellist) )
				printkeyinfo (reftime_for_list_key, dkp, oldpath);		/* print entry */
			oldpath = dkp->dname;
			dkp = dkp->next;
		}
	}
}
#endif

void	zkt_list_keys (time_t reftime, const dki_t *data)
{
#if ! defined(USE_TREE) || !USE_TREE
	const   dki_t   *dkp;
	const   char   *oldpath;
#endif

	if ( data )    /* print headline if list is not empty */
		printkeyinfo (reftime, NULL, "");

#if defined(USE_TREE) && USE_TREE
	reftime_for_list_key = reftime;
	twalk (data, list_key);
#else
	oldpath = "";
	for ( dkp = data; dkp; dkp = dkp->next )       /* loop through list */
	{
		if ( labellist == NULL || isinlist (dkp->name, labellist) )
			printkeyinfo (reftime, dkp, oldpath);            /* print entry */
		oldpath = dkp->dname;
	}
#endif
}

#if defined(USE_TREE) && USE_TREE
# if 0
static	void	list_trustedkey (const dki_t **nodep, const VISIT which, int depth)
{
	const	dki_t	*dkp;

	if ( nodep == NULL )
		return;

	dkp = *nodep;
	if ( which == INORDER || which == LEAF )
	{
// fprintf (stderr, "list_trustedkey order=%d(pre=0,in=1,post=2,leaf=3) depth=%d %s\n", which, depth, dkp->name);
		/* loop through list */
		while ( dkp )
		{
			if ( (dki_isksk (dkp) || zskflag) &&
			     (labellist == NULL || isinlist (dkp->name, labellist)) )
				dki_prt_trustedkey (dkp, stdout);
			dkp = dkp->next;
		}
	}
}
# else
const	dki_t	*parent;
static	void	list_trustedkey (const dki_t **nodep, const VISIT which, int depth)
{
	const	dki_t	*dkp;

	if ( nodep == NULL )
		return;

	dkp = *nodep;
	if ( which == INORDER || which == LEAF )
	{
// fprintf (stderr, "list_trustedkey order=%d(pre=0,in=1,post=2,leaf=3) depth=%d %s\n", which, depth, dkp->name);
		if ( labellist && !isinlist (dkp->name, labellist) )
			return;

		if ( parent == NULL || !issubdomain (dkp->name, parent->name) )
		{
			parent = dkp;	
			/* loop through list */
			while ( dkp )
			{
				if ( dki_isksk (dkp) || (zskflag && dki_iszsk (dkp)) )
					dki_prt_trustedkey (dkp, stdout);
				dkp = dkp->next;
			}
		}
	}
}
static	void	list_managedkey (const dki_t **nodep, const VISIT which, int depth)
{
	const	dki_t	*dkp;

	if ( nodep == NULL )
		return;

	dkp = *nodep;
	if ( which == INORDER || which == LEAF )
	{
// fprintf (stderr, "list_trustedkey order=%d(pre=0,in=1,post=2,leaf=3) depth=%d %s\n", which, depth, dkp->name);
		if ( labellist && !isinlist (dkp->name, labellist) )
			return;

		if ( parent == NULL || !issubdomain (dkp->name, parent->name) )
		{
			const	dki_t	*dkp_head = NULL;
			const	dki_t	*standby = NULL;

			parent = dkp;

			dkp_head = dkp;
			/* look for a standby key */
			for ( dkp = dkp_head; dkp; dkp = dkp->next )
				if ( dki_isksk (dkp) && dki_ispublished (dkp) )
					standby = dkp;

			if ( !standby )	/* no standby key found ? */
				return;

			/* print all non-standby ksk */
			for ( dkp = dkp_head; dkp; dkp = dkp->next )
				if ( dki_isksk (dkp) && dkp != standby )
					dki_prt_managedkey (dkp, stdout);
		}
	}
}
# endif
#endif

void	zkt_list_trustedkeys (const dki_t *data)
{

	/* print headline if list is not empty */
	if ( data && headerflag )
		printf ("trusted-keys {\n");

#if defined(USE_TREE) && USE_TREE
	twalk (data, list_trustedkey);
#else
	for ( dkp = data; dkp; dkp = dkp->next )	/* loop through list */
		if ( (dki_isksk (dkp) || (zskflag && dki_iszsk (dkp)) &&
		     (labellist == NULL || isinlist (dkp->name, labellist)) )
			dki_prt_trustedkey (dkp, stdout);
#endif

	/* print end of trusted-key section */
	if ( data && headerflag )
		printf ("};\n");
}

void	zkt_list_managedkeys (const dki_t *data)
{

	/* print headline if list is not empty */
	if ( data && headerflag )
		printf ("managed-keys {\n");

#if defined(USE_TREE) && USE_TREE
	twalk (data, list_managedkey);
#else
	for ( dkp = data; dkp; dkp = dkp->next )	/* loop through list */
		if ( (dki_isksk (dkp) || zskflag) &&
		     (labellist == NULL || isinlist (dkp->name, labellist)) )
			dki_prt_managedkey (dkp, stdout);
#endif

	/* print end of trusted-key section */
	if ( data && headerflag )
		printf ("};\n");
}

#if defined(USE_TREE) && USE_TREE
static	void	list_dnskey (const dki_t **nodep, const VISIT which, int depth)
{
	const	dki_t	*dkp;
	int	ksk;

	if ( nodep == NULL )
		return;

	if ( which == INORDER || which == LEAF )
		for ( dkp = *nodep; dkp; dkp = dkp->next )
		{
			ksk = dki_isksk (dkp);
			if ( (ksk && !kskflag) || (!ksk && !zskflag) )
				continue;

			if ( labellist == NULL || isinlist (dkp->name, labellist) )
			{
				if ( headerflag )
					dki_prt_comment (dkp, stdout);
				dki_prt_dnskey (dkp, stdout);
			}
		}
}
#endif

void	zkt_list_dnskeys (const dki_t *data)
{
#if defined(USE_TREE) && USE_TREE
	twalk (data, list_dnskey);
#else
	const	dki_t	*dkp;
	int	ksk;

	for ( dkp = data; dkp; dkp = dkp->next )
	{
		ksk = dki_isksk (dkp);
		if ( (ksk && !kskflag) || (!ksk && !zskflag) )
			continue;

		if ( labellist == NULL || isinlist (dkp->name, labellist) )
		{
			if ( headerflag )
				dki_prt_comment (dkp, stdout);
			dki_prt_dnskey (dkp, stdout);
		}
	}
#endif
}

#if defined(USE_TREE) && USE_TREE
static	const	dki_t	*searchresult;
static	int	searchitem;
static	void	tag_search (const dki_t **nodep, const VISIT which, int depth)
{
	const	dki_t	*dkp;

	if ( nodep == NULL )
		return;

	if ( which == PREORDER || which == LEAF )
		for ( dkp = *nodep; dkp; dkp = dkp->next )
		{
			if ( dkp->tag == searchitem )
			{
				if ( searchresult == NULL )
					searchresult = dkp;
				else
					searchitem = 0;
			}
		}
}
#endif
const	dki_t	*zkt_search (const dki_t *data, int searchtag, const char *keyname)
{
	const dki_t	*dkp = NULL;

#if defined(USE_TREE) && USE_TREE
	if ( keyname == NULL || *keyname == '\0' )
	{
		searchresult = NULL;
		searchitem = searchtag;
		twalk (data, tag_search);
		if ( searchresult != NULL && searchitem == 0 )
			dkp = (void *)01;
		else
			dkp = searchresult;
	}
	else
		dkp = (dki_t*)dki_tsearch (data, searchtag, keyname);
#else
	dkp = (dki_t*)dki_search (data, searchtag, keyname);
#endif
	return dkp;
}

